#ifndef __CHAR_SEQUENCE_ADAPTOR_H_
#define __CHAR_SEQUENCE_ADAPTOR_H_

#include <string>

namespace cli {

class CharSequenceBase {
	std::string putbackBuffer;

protected:
	const char * linePtr;
	bool endOfTransmition = false;
	bool endOfSequence = false;

	virtual void fetch ( bool readNextLine ) = 0;

public:
	CharSequenceBase ( ) = default;

	CharSequenceBase ( CharSequenceBase && ) noexcept = default;

	CharSequenceBase ( const CharSequenceBase & ) = delete;

	CharSequenceBase & operator = ( CharSequenceBase && ) = delete;

	CharSequenceBase & operator = ( const CharSequenceBase & ) = delete;

	virtual ~CharSequenceBase ( ) noexcept = default;

	int getCharacter ( ) const {
		if ( ! putbackBuffer.empty ( ) )
			return ( int ) putbackBuffer.back ( );
		else if ( * linePtr != 0 )
			return ( int ) * linePtr;
		else
			return EOF;
	}

	void advance ( bool readNextLine ) {
		if ( ! putbackBuffer.empty ( ) ) {
			putbackBuffer.pop_back ( );
		} else {
			if ( * linePtr != 0 )
				++ linePtr;
			if ( * linePtr == '\0' )
				fetch ( readNextLine );
		}
	}

	virtual std::string getLine ( ) const = 0;

	bool isEndOfTransmition ( ) const {
		return endOfTransmition;
	}

	bool isEndOfSequence ( ) const {
		return endOfSequence;
	}

	void putback ( std::string string ) {
		putbackBuffer.insert ( putbackBuffer.end ( ), string.rbegin ( ), string.rend ( ) );
	}
};

} /* namespace cli */

#endif /* __CHAR_SEQUENCE_ADAPTOR_H_ */
